namespace config
{
    template <typename T>
    T convert(const std::string& value);

    template <>
    float convert(const std::string& value) {
        try {
            return std::stof(value);
        }
        catch (const std::exception& e) {
            std::cerr << "Conversion error: " << e.what() << std::endl;
            return 0.0f; // Default value
        }
    }

    template <>
    bool convert(const std::string& value) {
        return value == "true";
    }


    void load_config() {
        std::ifstream configFile("C:\\Medium\\configs\\fivem.krptd");
        if (!configFile.is_open()) {
            std::cerr << "Failed to open config file for reading." << std::endl;
            return;
        }

        std::unordered_map<std::string, std::function<void(const std::string&)>> configMap = {
            {"ammobar_width", [](const std::string& value) { globals::ammobar_width = convert<float>(value); } },
            {"hit_sound_effect", [](const std::string& value) { globals::hit_sound_effect = convert<float>(value); } },
            {"kill_sound_effect", [](const std::string& value) { globals::kill_sound_effect = convert<float>(value); } },
            {"tracer_type", [](const std::string& value) { globals::tracer_type = convert<float>(value); } },
            {"bullet_impact_duration", [](const std::string& value) { globals::bullet_impact_duration = convert<float>(value); } },
            {"tracer_duration", [](const std::string& value) { globals::tracer_duration = convert<float>(value); } },
            {"smoothing.x", [](const std::string& value) { globals::smoothing.x = convert<float>(value); } },
            {"smoothing.y", [](const std::string& value) { globals::smoothing.y = convert<float>(value); } },
            {"npc_identify_type", [](const std::string& value) { globals::npc_identify_type = convert<float>(value); } },
            {"recoil_type", [](const std::string& value) { globals::recoil_type = convert<float>(value); } },
            {"spread_type", [](const std::string& value) { globals::spread_type = convert<float>(value); } },
            {"tracer_color[0]", [](const std::string& value) { globals::tracer_color[0] = convert<float>(value); } },
            {"tracer_color[1]", [](const std::string& value) { globals::tracer_color[1] = convert<float>(value); } },
            {"tracer_color[2]", [](const std::string& value) { globals::tracer_color[2] = convert<float>(value); } },
            {"ammobar_color[0]", [](const std::string& value) { globals::ammobar_color[0] = convert<float>(value); } },
            {"ammobar_color[1]", [](const std::string& value) { globals::ammobar_color[1] = convert<float>(value); } },
            {"ammobar_color[2]", [](const std::string& value) { globals::ammobar_color[2] = convert<float>(value); } },
            {"bullet_impact_circle_color[0]", [](const std::string& value) { globals::bullet_impact_circle_color[0] = convert<float>(value); } },
            {"bullet_impact_circle_color[1]", [](const std::string& value) { globals::bullet_impact_circle_color[1] = convert<float>(value); } },
            {"bullet_impact_circle_color[2]", [](const std::string& value) { globals::bullet_impact_circle_color[2] = convert<float>(value); } },
            {"damage_number_color[0]", [](const std::string& value) { globals::damage_number_color[0] = convert<float>(value); } },
            {"damage_number_color[1]", [](const std::string& value) { globals::damage_number_color[1] = convert<float>(value); } },
            {"damage_number_color[2]", [](const std::string& value) { globals::damage_number_color[2] = convert<float>(value); } },
            {"f_accent_color[0]", [](const std::string& value) { globals::f_accent_color[0] = convert<float>(value); } },
            {"f_accent_color[1]", [](const std::string& value) { globals::f_accent_color[1] = convert<float>(value); } },
            {"f_accent_color[2]", [](const std::string& value) { globals::f_accent_color[2] = convert<float>(value); } },
            {"bullet_impact_hitmarker_color[0]", [](const std::string& value) { globals::bullet_impact_hitmarker_color[0] = convert<float>(value); } },
            {"bullet_impact_hitmarker_color[1]", [](const std::string& value) { globals::bullet_impact_hitmarker_color[1] = convert<float>(value); } },
            {"bullet_impact_hitmarker_color[2]", [](const std::string& value) { globals::bullet_impact_hitmarker_color[2] = convert<float>(value); } },
            {"particle_hit_color[0]", [](const std::string& value) { globals::particle_hit_color[0] = convert<float>(value); } },
            {"particle_hit_color[1]", [](const std::string& value) { globals::particle_hit_color[1] = convert<float>(value); } },
            {"particle_hit_color[2]", [](const std::string& value) { globals::particle_hit_color[2] = convert<float>(value); } },
            {"closest_bones_filter[0]", [](const std::string& value) { globals::closest_bones_filter[0] = convert<bool>(value); } },
            {"closest_bones_filter[1]", [](const std::string& value) { globals::closest_bones_filter[1] = convert<bool>(value); } },
            {"closest_bones_filter[2]", [](const std::string& value) { globals::closest_bones_filter[2] = convert<bool>(value); } },
            {"closest_bones_filter[3]", [](const std::string& value) { globals::closest_bones_filter[3] = convert<bool>(value); } },
            {"closest_bones_filter[4]", [](const std::string& value) { globals::closest_bones_filter[4] = convert<bool>(value); } },
            {"closest_bones_filter[5]", [](const std::string& value) { globals::closest_bones_filter[5] = convert<bool>(value); } },
            {"closest_bones_filter[6]", [](const std::string& value) { globals::closest_bones_filter[6] = convert<bool>(value); } },
            {"closest_bones_filter[7]", [](const std::string& value) { globals::closest_bones_filter[7] = convert<bool>(value); } },
            {"closest_bones_filter[8]", [](const std::string& value) { globals::closest_bones_filter[8] = convert<bool>(value); } },
    {"toggles.draw_snaplines", [](const std::string& value) { globals::toggles::draw_snaplines = (value == "true"); }},
    {"toggles.hit_sound", [](const std::string& value) { globals::toggles::hit_sound = (value == "true"); }},
    {"toggles.kill_sound", [](const std::string& value) { globals::toggles::kill_sound = (value == "true"); }},
    {"toggles.hit_particles", [](const std::string& value) { globals::toggles::hit_particles = (value == "true"); }},
    {"toggles.streamproof", [](const std::string& value) { globals::toggles::streamproof = (value == "true"); }},
    {"toggles.spotify_widget", [](const std::string& value) { globals::toggles::spotify_widget = (value == "true"); }},
    {"toggles.ammobar", [](const std::string& value) { globals::toggles::ammobar = (value == "true"); }},
    {"toggles.draw_watermark", [](const std::string& value) { globals::toggles::draw_watermark = (value == "true"); }},
    {"toggles.discord_rpc", [](const std::string& value) { globals::toggles::discord_rpc = (value == "true"); }},
    {"toggles.draw_box", [](const std::string& value) { globals::toggles::draw_box = (value == "true"); }},
    {"toggles.draw_filled_box", [](const std::string& value) { globals::toggles::draw_filled_box = (value == "true"); }},
    {"toggles.draw_skeleton", [](const std::string& value) { globals::toggles::draw_skeleton = (value == "true"); }},
    {"toggles.draw_joints", [](const std::string& value) { globals::toggles::draw_joints = (value == "true"); }},
    {"toggles.draw_healthbar", [](const std::string& value) { globals::toggles::draw_healthbar = (value == "true"); }},
    {"toggles.draw_health", [](const std::string& value) { globals::toggles::draw_health = (value == "true"); }},
    {"toggles.draw_armorbar", [](const std::string& value) { globals::toggles::draw_armorbar = (value == "true"); }},
    {"toggles.draw_armor", [](const std::string& value) { globals::toggles::draw_armor = (value == "true"); }},
    {"toggles.draw_fov", [](const std::string& value) { globals::toggles::draw_fov = (value == "true"); }},
    {"toggles.draw_filled_fov", [](const std::string& value) { globals::toggles::draw_filled_fov = (value == "true"); }},
    {"toggles.draw_distance", [](const std::string& value) { globals::toggles::draw_distance = (value == "true"); }},
    {"toggles.ignore_npc", [](const std::string& value) { globals::toggles::ignore_npc = (value == "true"); }},
    {"toggles.ignore_own_vehicle", [](const std::string& value) { globals::toggles::ignore_own_vehicle = (value == "true"); }},
    {"toggles.ignore_self_esp", [](const std::string& value) { globals::toggles::ignore_self_esp = (value == "true"); }},
    {"toggles.ignore_dead_esp", [](const std::string& value) { globals::toggles::ignore_dead_esp = (value == "true"); }},
    {"toggles.ignore_dead_aim", [](const std::string& value) { globals::toggles::ignore_dead_aim = (value == "true"); }},
    {"toggles.ignore_npc_aim", [](const std::string& value) { globals::toggles::ignore_npc_aim = (value == "true"); }},
    {"toggles.ignore_friend_aim", [](const std::string& value) { globals::toggles::ignore_friend_aim = (value == "true"); }},
    {"toggles.ignore_friend_esp", [](const std::string& value) { globals::toggles::ignore_friend_esp = (value == "true"); }},
    {"toggles.aimbot", [](const std::string& value) { globals::toggles::aimbot = (value == "true"); }},
    {"toggles.magic_bullet", [](const std::string& value) { globals::toggles::magic_bullet = (value == "true"); }},
    {"toggles.draw_target_line", [](const std::string& value) { globals::toggles::draw_target_line = (value == "true"); }},
    {"toggles.external_overlay", [](const std::string& value) { globals::toggles::external_overlay = (value == "true"); }},
    {"toggles.draw_weapon_name", [](const std::string& value) { globals::toggles::draw_weapon_name = (value == "true"); }},
    {"toggles.draw_player_name", [](const std::string& value) { globals::toggles::draw_player_name = (value == "true"); }},
    {"toggles.draw_player_info", [](const std::string& value) { globals::toggles::draw_player_info = (value == "true"); }},
    {"toggles.draw_crosshair", [](const std::string& value) { globals::toggles::draw_crosshair = (value == "true"); }},
    {"toggles.heal_key", [](const std::string& value) { globals::toggles::heal_key = (value == "true"); }},
    {"toggles.noclip", [](const std::string& value) { globals::toggles::noclip = (value == "true"); }},
    {"toggles.draw_vehicle_names", [](const std::string& value) { globals::toggles::draw_vehicle_names = (value == "true"); }},
    {"toggles.zoom_key", [](const std::string& value) { globals::toggles::zoom_key = (value == "true"); }},
    {"toggles.draw_vehicle_distance", [](const std::string& value) { globals::toggles::draw_vehicle_distance = (value == "true"); }},
    {"toggles.hide_bar_if_zero", [](const std::string& value) { globals::toggles::hide_bar_if_zero = (value == "true"); }},
    {"toggles.draw_driver_status", [](const std::string& value) { globals::toggles::draw_driver_status = (value == "true"); }},
    {"toggles.draw_lock_status", [](const std::string& value) { globals::toggles::draw_lock_status = (value == "true"); }},
    {"toggles.draw_healthbar_vehicle", [](const std::string& value) { globals::toggles::draw_healthbar_vehicle = (value == "true"); }},
    {"toggles.tracers", [](const std::string& value) { globals::toggles::tracers = (value == "true"); }},
    {"toggles.damage_numbers", [](const std::string& value) { globals::toggles::damage_numbers = (value == "true"); }},
    {"toggles.bullet_impact_circle", [](const std::string& value) { globals::toggles::bullet_impact_circle = (value == "true"); }},
    {"toggles.bullet_impact_hitmarker", [](const std::string& value) { globals::toggles::bullet_impact_hitmarker = (value == "true"); }},
    {"toggles.draw_net_id", [](const std::string& value) { globals::toggles::draw_net_id = (value == "true"); }},
    {"toggles.players_looking_warning", [](const std::string& value) { globals::toggles::players_looking_warning = (value == "true"); }},
    {"toggles.draw_oof_indicators", [](const std::string& value) { globals::toggles::draw_oof_indicators = (value == "true"); }},
    {"toggles.ignore_without_weapon_aim", [](const std::string& value) { globals::toggles::ignore_without_weapon_aim = (value == "true"); }},
    {"toggles.friend_marker", [](const std::string& value) { globals::toggles::friend_marker = (value == "true"); }},
    {"toggles.target_marker", [](const std::string& value) { globals::toggles::target_marker = (value == "true"); }},
        { "toggles.text_toggles[0]", [](const std::string& value) { globals::toggles::text_toggles[0] = convert<bool>(value); } },
        { "toggles.text_toggles[1]", [](const std::string& value) { globals::toggles::text_toggles[1] = convert<bool>(value); } },
        { "toggles.text_toggles[2]", [](const std::string& value) { globals::toggles::text_toggles[2] = convert<bool>(value); } },
        { "toggles.text_toggles[3]", [](const std::string& value) { globals::toggles::text_toggles[3] = convert<bool>(value); } },
        { "toggles.text_toggles[4]", [](const std::string& value) { globals::toggles::text_toggles[4] = convert<bool>(value); } },
        {"esp.max_distance", [](const std::string& value) { globals::esp::max_distance = std::stof(value); } },
        {"esp.vehicle_max_distance", [](const std::string& value) { globals::esp::vehicle_max_distance = std::stof(value); } },
        {"esp.max_distance_text", [](const std::string& value) { globals::esp::max_distance_text = std::stof(value); } },
        {"esp.font", [](const std::string& value) { globals::esp::font = std::stof(value); } },
        {"esp.bar_position", [](const std::string& value) { globals::esp::bar_position = std::stof(value); } },
        {"esp.text_position", [](const std::string& value) { globals::esp::text_position = std::stof(value); } },
        {"esp.text_position_vehicle", [](const std::string& value) { globals::esp::text_position_vehicle = std::stof(value); } },
        {"esp.box_type", [](const std::string& value) { globals::esp::box_type = std::stof(value); } },
        {"esp.filled_box_type", [](const std::string& value) { globals::esp::filled_box_type = std::stof(value); } },
        {"esp.skeleton_type", [](const std::string& value) { globals::esp::skeleton_type = std::stof(value); } },
        {"esp.box_color[0]", [](const std::string& value) { globals::esp::box_color[0] = std::stof(value); } },
        {"esp.box_color[1]", [](const std::string& value) { globals::esp::box_color[1] = std::stof(value); } },
        {"esp.box_color[2]", [](const std::string& value) { globals::esp::box_color[2] = std::stof(value); } },
        {"esp.filled_box_color[0]", [](const std::string& value) { globals::esp::filled_box_color[0] = std::stof(value); } },
        {"esp.filled_box_color[1]", [](const std::string& value) { globals::esp::filled_box_color[1] = std::stof(value); } },
        {"esp.filled_box_color[2]", [](const std::string& value) { globals::esp::filled_box_color[2] = std::stof(value); } },
        {"esp.filled_box_color[3]", [](const std::string& value) { globals::esp::filled_box_color[3] = std::stof(value); } },
        {"esp.oof_indicator_color[0]", [](const std::string& value) { globals::esp::oof_indicator_color[0] = std::stof(value); } },
        {"esp.oof_indicator_color[1]", [](const std::string& value) { globals::esp::oof_indicator_color[1] = std::stof(value); } },
        {"esp.oof_indicator_color[2]", [](const std::string& value) { globals::esp::oof_indicator_color[2] = std::stof(value); } },
        {"esp.oof_indicator_color[3]", [](const std::string& value) { globals::esp::oof_indicator_color[3] = std::stof(value); } },
        {"esp.snaplines_color[0]", [](const std::string& value) { globals::esp::snaplines_color[0] = std::stof(value); } },
        {"esp.snaplines_color[1]", [](const std::string& value) { globals::esp::snaplines_color[1] = std::stof(value); } },
        {"esp.snaplines_color[2]", [](const std::string& value) { globals::esp::snaplines_color[2] = std::stof(value); } },
        {"esp.skeleton_color[0]", [](const std::string& value) { globals::esp::skeleton_color[0] = std::stof(value); } },
        {"esp.skeleton_color[1]", [](const std::string& value) { globals::esp::skeleton_color[1] = std::stof(value); } },
        {"esp.skeleton_color[2]", [](const std::string& value) { globals::esp::skeleton_color[2] = std::stof(value); } },
        {"esp.skeleton_color[3]", [](const std::string& value) { globals::esp::skeleton_color[3] = std::stof(value); } },
            { "esp.joints_color[0]", [](const std::string& value) { globals::esp::joints_color[0] = std::stof(value); } },
            { "esp.joints_color[1]", [](const std::string& value) { globals::esp::joints_color[1] = std::stof(value); } },
            { "esp.joints_color[2]", [](const std::string& value) { globals::esp::joints_color[2] = std::stof(value); } },
            { "esp.joints_color[3]", [](const std::string& value) { globals::esp::joints_color[3] = std::stof(value); } },
        {"esp.healthbar_color[0]", [](const std::string& value) { globals::esp::healthbar_color[0] = std::stof(value); } },
        {"esp.healthbar_color[1]", [](const std::string& value) { globals::esp::healthbar_color[1] = std::stof(value); } },
        {"esp.healthbar_color[2]", [](const std::string& value) { globals::esp::healthbar_color[2] = std::stof(value); } },
        {"esp.armorbar_color[0]", [](const std::string& value) { globals::esp::armorbar_color[0] = std::stof(value); } },
        {"esp.armorbar_color[1]", [](const std::string& value) { globals::esp::armorbar_color[1] = std::stof(value); } },
        {"esp.armorbar_color[2]", [](const std::string& value) { globals::esp::armorbar_color[2] = std::stof(value); } },
        {"crosshair_color[0]", [](const std::string& value) { globals::crosshair_color[0] = std::stof(value); } },
        {"crosshair_color[1]", [](const std::string& value) { globals::crosshair_color[1] = std::stof(value); } },
        {"crosshair_color[2]", [](const std::string& value) { globals::crosshair_color[2] = std::stof(value); } },
        {"crosshair_color[3]", [](const std::string& value) { globals::crosshair_color[3] = std::stof(value); } },
        {"fov_color[0]", [](const std::string& value) { globals::fov_color[0] = std::stof(value); } },
        {"fov_color[1]", [](const std::string& value) { globals::fov_color[1] = std::stof(value); } },
        {"fov_color[2]", [](const std::string& value) { globals::fov_color[2] = std::stof(value); } },
        {"filled_fov_color[0]", [](const std::string& value) { globals::filled_fov_color[0] = std::stof(value); } },
        {"filled_fov_color[1]", [](const std::string& value) { globals::filled_fov_color[1] = std::stof(value); } },
        {"filled_fov_color[2]", [](const std::string& value) { globals::filled_fov_color[2] = std::stof(value); } },
        {"filled_fov_color[3]", [](const std::string& value) { globals::filled_fov_color[3] = std::stof(value); } },
        {"esp.hat_color[0]", [](const std::string& value) { globals::esp::hat_color[0] = std::stof(value); } },
        {"esp.hat_color[1]", [](const std::string& value) { globals::esp::hat_color[1] = std::stof(value); } },
        {"esp.hat_color[2]", [](const std::string& value) { globals::esp::hat_color[2] = std::stof(value); } },
        {"esp.hat_color[3]", [](const std::string& value) { globals::esp::hat_color[3] = std::stof(value); } },
        {"esp.friend_filled_box_color[0]", [](const std::string& value) { globals::esp::friend_filled_box_color[0] = std::stof(value); } },
        {"esp.friend_filled_box_color[1]", [](const std::string& value) { globals::esp::friend_filled_box_color[1] = std::stof(value); } },
        {"esp.friend_filled_box_color[2]", [](const std::string& value) { globals::esp::friend_filled_box_color[2] = std::stof(value); } },
        {"esp.friend_filled_box_color[3]", [](const std::string& value) { globals::esp::friend_filled_box_color[3] = std::stof(value); } },
        {"esp.target_filled_box_color[0]", [](const std::string& value) { globals::esp::target_filled_box_color[0] = std::stof(value); } },
        {"esp.target_filled_box_color[1]", [](const std::string& value) { globals::esp::target_filled_box_color[1] = std::stof(value); } },
        {"esp.target_filled_box_color[2]", [](const std::string& value) { globals::esp::target_filled_box_color[2] = std::stof(value); } },
        {"esp.target_filled_box_color[3]", [](const std::string& value) { globals::esp::target_filled_box_color[3] = std::stof(value); } },
        {"esp.friend_marker_color[0]", [](const std::string& value) { globals::esp::friend_marker_color[0] = std::stof(value); } },
        {"esp.friend_marker_color[1]", [](const std::string& value) { globals::esp::friend_marker_color[1] = std::stof(value); } },
        {"esp.friend_marker_color[2]", [](const std::string& value) { globals::esp::friend_marker_color[2] = std::stof(value); } },
        {"esp.friend_marker_color[3]", [](const std::string& value) { globals::esp::friend_marker_color[3] = std::stof(value); } },
        {"esp.target_marker_color[0]", [](const std::string& value) { globals::esp::target_marker_color[0] = std::stof(value); } },
        {"esp.target_marker_color[1]", [](const std::string& value) { globals::esp::target_marker_color[1] = std::stof(value); } },
        {"esp.target_marker_color[2]", [](const std::string& value) { globals::esp::target_marker_color[2] = std::stof(value); } },
        {"esp.target_marker_color[3]", [](const std::string& value) { globals::esp::target_marker_color[3] = std::stof(value); } },
        {"esp.friend_skeleton_color[0]", [](const std::string& value) { globals::esp::friend_skeleton_color[0] = std::stof(value); } },
        {"esp.friend_skeleton_color[1]", [](const std::string& value) { globals::esp::friend_skeleton_color[1] = std::stof(value); } },
        {"esp.friend_skeleton_color[2]", [](const std::string& value) { globals::esp::friend_skeleton_color[2] = std::stof(value); } },
        {"esp.friend_skeleton_color[3]", [](const std::string& value) { globals::esp::friend_skeleton_color[3] = std::stof(value); } },
    { "esp.friend_joints_color[0]", [](const std::string& value) { globals::esp::friend_joints_color[0] = std::stof(value); } },
    { "esp.friend_joints_color[1]", [](const std::string& value) { globals::esp::friend_joints_color[1] = std::stof(value); } },
    { "esp.friend_joints_color[2]", [](const std::string& value) { globals::esp::friend_joints_color[2] = std::stof(value); } },
    { "esp.friend_joints_color[3]", [](const std::string& value) { globals::esp::friend_joints_color[3] = std::stof(value); } },
        {"esp.target_skeleton_color[0]", [](const std::string& value) { globals::esp::target_skeleton_color[0] = std::stof(value); } },
        {"esp.target_skeleton_color[1]", [](const std::string& value) { globals::esp::target_skeleton_color[1] = std::stof(value); } },
        {"esp.target_skeleton_color[2]", [](const std::string& value) { globals::esp::target_skeleton_color[2] = std::stof(value); } },
        {"esp.target_skeleton_color[3]", [](const std::string& value) { globals::esp::target_skeleton_color[3] = std::stof(value); } },
    { "esp.target_joints_color[0]", [](const std::string& value) { globals::esp::target_joints_color[0] = std::stof(value); } },
    { "esp.target_joints_color[1]", [](const std::string& value) { globals::esp::target_joints_color[1] = std::stof(value); } },
    { "esp.target_joints_color[2]", [](const std::string& value) { globals::esp::target_joints_color[2] = std::stof(value); } },
    { "esp.target_joints_color[3]", [](const std::string& value) { globals::esp::target_joints_color[3] = std::stof(value); } },
        {"esp.friend_box_color[0]", [](const std::string& value) { globals::esp::friend_box_color[0] = std::stof(value); } },
        {"esp.friend_box_color[1]", [](const std::string& value) { globals::esp::friend_box_color[1] = std::stof(value); } },
        {"esp.friend_box_color[2]", [](const std::string& value) { globals::esp::friend_box_color[2] = std::stof(value); } },
        {"esp.target_box_color[0]", [](const std::string& value) { globals::esp::target_box_color[0] = std::stof(value); } },
        {"esp.target_box_color[1]", [](const std::string& value) { globals::esp::target_box_color[1] = std::stof(value); } },
        {"esp.target_box_color[2]", [](const std::string& value) { globals::esp::target_box_color[2] = std::stof(value); } },
        {"keys.aim_key", [](const std::string& value) { globals::keys::aim_key = std::stof(value); } },
        {"keys.heal_key", [](const std::string& value) { globals::keys::heal_key = std::stof(value); } },
        {"keys.menu_key", [](const std::string& value) { globals::keys::menu_key = std::stof(value); } },
        {"keys.exit_key", [](const std::string& value) { globals::keys::exit_key = std::stof(value); } },
        {"keys.noclip_key", [](const std::string& value) { globals::keys::noclip_key = std::stof(value); } },
        {"keys.force_body_key", [](const std::string& value) { globals::keys::force_body_key = std::stof(value); } },
        {"keys.magic_bullet_key", [](const std::string& value) { globals::keys::magic_bullet_key = std::stof(value); } },
        {"keys.fire_ammo_key", [](const std::string& value) { globals::keys::fire_ammo_key = std::stof(value); } },
        {"keys.explosive_ammo_key", [](const std::string& value) { globals::keys::explosive_ammo_key = std::stof(value); } },
        {"keys.explosive_melee_key", [](const std::string& value) { globals::keys::explosive_melee_key = std::stof(value); } }
        };


        std::string line;
        while (std::getline(configFile, line)) {
            std::istringstream iss(line);
            std::string key;

            if (std::getline(iss, key, '=')) {
                std::string value;
                if (std::getline(iss, value)) {
                    auto it = configMap.find(key);
                    if (it != configMap.end()) {
                        it->second(value);
                    }
                }
            }
        }
    }

    void save_config() {
        std::ofstream configFile("C:\\Medium\\configs\\fivem.krptd");
        if (!configFile.is_open()) {
            std::cerr << "Failed to open config file for writing." << std::endl;
            return;
        }

        configFile << std::boolalpha;

        configFile << "toggles.discord_rpc=" << globals::toggles::discord_rpc << std::endl;
        for (int i = 0; i < 9; ++i) {
            configFile << "closest_bones_filter[" << i << "]=" << globals::closest_bones_filter[i] << std::endl;
        }
        configFile << "fov=" << globals::fov << std::endl;
        configFile << "hit_sound_effect=" << globals::hit_sound_effect << std::endl;
        configFile << "kill_sound_effect=" << globals::kill_sound_effect << std::endl;
        configFile << "ammobar_width=" << globals::ammobar_width << std::endl;
        configFile << "tracer_type=" << globals::tracer_type << std::endl;
        for (int i = 0; i < 3; ++i) {
            configFile << "tracer_color[" << i << "]=" << globals::tracer_color[i] << std::endl;
        }
        for (int i = 0; i < 3; ++i) {
            configFile << "ammobar_color[" << i << "]=" << globals::ammobar_color[i] << std::endl;
        }
        for (int i = 0; i < 3; ++i) {
            configFile << "damage_number_color[" << i << "]=" << globals::damage_number_color[i] << std::endl;
        }
        for (int i = 0; i < 3; ++i) {
            configFile << "bullet_impact_circle_color[" << i << "]=" << globals::bullet_impact_circle_color[i] << std::endl;
        }
        for (int i = 0; i < 3; ++i) {
            configFile << "bullet_impact_hitmarker_color[" << i << "]=" << globals::bullet_impact_hitmarker_color[i] << std::endl;
        }
        for (int i = 0; i < 3; ++i) {
            configFile << "f_accent_color[" << i << "]=" << globals::f_accent_color[i] << std::endl;
        }
        for (int i = 0; i < 3; ++i) {
            configFile << "particle_hit_color[" << i << "]=" << globals::particle_hit_color[i] << std::endl;
        }
        configFile << "smoothing.x=" << globals::smoothing.x << std::endl;
        configFile << "smoothing.y=" << globals::smoothing.y << std::endl;
        configFile << "npc_identify_type=" << globals::npc_identify_type << std::endl;
        configFile << "spread_type=" << globals::spread_type << std::endl;
        configFile << "recoil_type=" << globals::recoil_type << std::endl;
        configFile << "tracer_duration=" << globals::tracer_duration << std::endl;
        configFile << "bullet_impact_duration=" << globals::bullet_impact_duration << std::endl;
        configFile << "keys.aim_key=" << globals::keys::aim_key << std::endl;
        configFile << "keys.heal_key=" << globals::keys::heal_key << std::endl;
        configFile << "keys.menu_key=" << globals::keys::menu_key << std::endl;
        configFile << "keys.exit_key=" << globals::keys::exit_key << std::endl;
        configFile << "keys.noclip_key=" << globals::keys::noclip_key << std::endl;
        configFile << "keys.force_body_key=" << globals::keys::force_body_key << std::endl;
        configFile << "keys.magic_bullet_key=" << globals::keys::magic_bullet_key << std::endl;
        configFile << "keys.explosive_ammo_key=" << globals::keys::explosive_ammo_key << std::endl;
        configFile << "keys.explosive_melee_key=" << globals::keys::explosive_melee_key << std::endl;
        configFile << "keys.fire_ammo_key=" << globals::keys::fire_ammo_key << std::endl;
        configFile << "toggles.draw_snaplines=" << globals::toggles::draw_snaplines << std::endl;
        configFile << "toggles.spotify_widget=" << globals::toggles::spotify_widget << std::endl;
        configFile << "toggles.streamproof=" << globals::toggles::streamproof << std::endl;
        configFile << "toggles.draw_watermark=" << globals::toggles::draw_watermark << std::endl;
        configFile << "toggles.draw_box=" << globals::toggles::draw_box << std::endl;
        configFile << "toggles.draw_filled_box=" << globals::toggles::draw_filled_box << std::endl;
        configFile << "toggles.draw_skeleton=" << globals::toggles::draw_skeleton << std::endl;
        configFile << "toggles.draw_joints=" << globals::toggles::draw_joints << std::endl;

        configFile << "toggles.draw_healthbar=" << globals::toggles::draw_healthbar << std::endl;
        configFile << "toggles.draw_health=" << globals::toggles::draw_health << std::endl;
        configFile << "toggles.draw_armorbar=" << globals::toggles::draw_armorbar << std::endl;
        configFile << "toggles.draw_armor=" << globals::toggles::draw_armor << std::endl;
        configFile << "toggles.draw_fov=" << globals::toggles::draw_fov << std::endl;
        configFile << "toggles.draw_filled_fov=" << globals::toggles::draw_filled_fov << std::endl;
        configFile << "toggles.draw_distance=" << globals::toggles::draw_distance << std::endl;
        configFile << "toggles.ignore_npc=" << globals::toggles::ignore_npc << std::endl;
        configFile << "toggles.ignore_own_vehicle=" << globals::toggles::ignore_own_vehicle << std::endl;
        configFile << "toggles.ignore_self_esp=" << globals::toggles::ignore_self_esp << std::endl;
        configFile << "toggles.ignore_dead_esp=" << globals::toggles::ignore_dead_esp << std::endl;
        configFile << "toggles.ignore_dead_aim=" << globals::toggles::ignore_dead_aim << std::endl;
        configFile << "toggles.ignore_npc_aim=" << globals::toggles::ignore_npc_aim << std::endl;
        configFile << "toggles.ignore_friend_aim=" << globals::toggles::ignore_friend_aim << std::endl;
        configFile << "toggles.ignore_friend_esp=" << globals::toggles::ignore_friend_esp << std::endl;
        configFile << "toggles.aimbot=" << globals::toggles::aimbot << std::endl;
        configFile << "toggles.magic_bullet=" << globals::toggles::magic_bullet << std::endl;
        configFile << "toggles.draw_target_line=" << globals::toggles::draw_target_line << std::endl;
        configFile << "toggles.external_overlay=" << globals::toggles::external_overlay << std::endl;
        configFile << "toggles.draw_weapon_name=" << globals::toggles::draw_weapon_name << std::endl;
        configFile << "toggles.draw_player_name=" << globals::toggles::draw_player_name << std::endl;
        configFile << "toggles.draw_player_info=" << globals::toggles::draw_player_info << std::endl;
        configFile << "toggles.draw_crosshair=" << globals::toggles::draw_crosshair << std::endl;
        configFile << "toggles.heal_key=" << globals::toggles::heal_key << std::endl;
        configFile << "toggles.noclip=" << globals::toggles::noclip << std::endl;
        configFile << "toggles.draw_vehicle_names=" << globals::toggles::draw_vehicle_names << std::endl;
        configFile << "toggles.zoom_key=" << globals::toggles::zoom_key << std::endl;
        configFile << "toggles.draw_vehicle_distance=" << globals::toggles::draw_vehicle_distance << std::endl;
        configFile << "toggles.hide_bar_if_zero=" << globals::toggles::hide_bar_if_zero << std::endl;
        configFile << "toggles.draw_driver_status=" << globals::toggles::draw_driver_status << std::endl;
        configFile << "toggles.draw_lock_status=" << globals::toggles::draw_lock_status << std::endl;
        configFile << "toggles.draw_healthbar_vehicle=" << globals::toggles::draw_healthbar_vehicle << std::endl;
        configFile << "toggles.tracers=" << globals::toggles::tracers << std::endl;
        configFile << "toggles.bullet_impact_circle=" << globals::toggles::bullet_impact_circle << std::endl;
        configFile << "toggles.hit_particles=" << globals::toggles::hit_particles << std::endl;
        configFile << "toggles.bullet_impact_hitmarker=" << globals::toggles::bullet_impact_hitmarker << std::endl;
        configFile << "toggles.friend_marker=" << globals::toggles::friend_marker << std::endl;
        configFile << "toggles.target_marker=" << globals::toggles::target_marker << std::endl;
        configFile << "toggles.damage_numbers=" << globals::toggles::damage_numbers << std::endl;
        configFile << "toggles.hit_sound=" << globals::toggles::hit_sound << std::endl;
        configFile << "toggles.kill_sound=" << globals::toggles::kill_sound << std::endl;
        for (int i = 0; i < 6; ++i) {
            configFile << "toggles.text_toggles[" << i << "]=" << globals::toggles::text_toggles[i] << std::endl;
        }
        configFile << "toggles.draw_net_id=" << globals::toggles::draw_net_id << std::endl;
        configFile << "toggles.players_looking_warning=" << globals::toggles::players_looking_warning << std::endl;
        configFile << "toggles.draw_oof_indicators=" << globals::toggles::draw_oof_indicators << std::endl;
        configFile << "toggles.ignore_without_weapon_aim=" << globals::toggles::ignore_without_weapon_aim << std::endl;
        configFile << "toggles.ammobar=" << globals::toggles::ammobar << std::endl;

        configFile << std::fixed << std::setprecision(2);

        configFile << "esp.max_distance=" << globals::esp::max_distance << std::endl;
        configFile << "esp.vehicle_max_distance=" << globals::esp::vehicle_max_distance << std::endl;
        configFile << "esp.max_distance_text=" << globals::esp::max_distance_text << std::endl;
        configFile << "esp.font=" << globals::esp::font << std::endl;
        configFile << "esp.bar_position=" << globals::esp::bar_position << std::endl;
        configFile << "esp.text_position=" << globals::esp::text_position << std::endl;
        configFile << "esp.box_type=" << globals::esp::box_type << std::endl;
        configFile << "esp.filled_box_type=" << globals::esp::filled_box_type << std::endl;
        configFile << "esp.skeleton_type=" << globals::esp::skeleton_type << std::endl;
        configFile << "esp.text_position_vehicle=" << globals::esp::text_position_vehicle << std::endl;
        for (int i = 0; i < 3; ++i) {
            configFile << "esp.snaplines_color[" << i << "]=" << globals::esp::snaplines_color[i] << std::endl;
        }
        for (int i = 0; i < 3; ++i) {
            configFile << "esp.box_color[" << i << "]=" << globals::esp::box_color[i] << std::endl;
        }
        for (int i = 0; i < 4; ++i) {
            configFile << "esp.filled_box_color[" << i << "]=" << globals::esp::filled_box_color[i] << std::endl;
        }
        for (int i = 0; i < 4; ++i) {
            configFile << "esp.skeleton_color[" << i << "]=" << globals::esp::skeleton_color[i] << std::endl;
        }
        for (int i = 0; i < 4; ++i) {
            configFile << "esp.joints_color[" << i << "]=" << globals::esp::joints_color[i] << std::endl;
        }
        for (int i = 0; i < 3; ++i) {
            configFile << "esp.healthbar_color[" << i << "]=" << globals::esp::healthbar_color[i] << std::endl;
        }
        for (int i = 0; i < 3; ++i) {
            configFile << "esp.armorbar_color[" << i << "]=" << globals::esp::armorbar_color[i] << std::endl;
        }
        for (int i = 0; i < 4; ++i) {
            configFile << "esp.crosshair_color[" << i << "]=" << globals::crosshair_color[i] << std::endl;
        }
        for (int i = 0; i < 3; ++i) {
            configFile << "esp.fov_color[" << i << "]=" << globals::fov_color[i] << std::endl;
        }
        for (int i = 0; i < 4; ++i) {
            configFile << "esp.filled_fov_color[" << i << "]=" << globals::filled_fov_color[i] << std::endl;
        }
        for (int i = 0; i < 4; ++i) {
            configFile << "esp.hat_color[" << i << "]=" << globals::esp::hat_color[i] << std::endl;
        }
        for (int i = 0; i < 4; ++i) {
            configFile << "esp.friend_skeleton_color[" << i << "]=" << globals::esp::friend_skeleton_color[i] << std::endl;
        }
        for (int i = 0; i < 4; ++i) {
            configFile << "esp.friend_joints_color[" << i << "]=" << globals::esp::friend_joints_color[i] << std::endl;
        }
        for (int i = 0; i < 4; ++i) {
            configFile << "esp.target_skeleton_color[" << i << "]=" << globals::esp::target_skeleton_color[i] << std::endl;
        }
        for (int i = 0; i < 4; ++i) {
            configFile << "esp.target_joints_color[" << i << "]=" << globals::esp::target_joints_color[i] << std::endl;
        }
        for (int i = 0; i < 4; ++i) {
            configFile << "esp.friend_filled_box_color[" << i << "]=" << globals::esp::friend_filled_box_color[i] << std::endl;
        }
        for (int i = 0; i < 4; ++i) {
            configFile << "esp.target_filled_box_color[" << i << "]=" << globals::esp::target_filled_box_color[i] << std::endl;
        }
        for (int i = 0; i < 4; ++i) {
            configFile << "esp.friend_marker_color[" << i << "]=" << globals::esp::friend_marker_color[i] << std::endl;
        }
        for (int i = 0; i < 4; ++i) {
            configFile << "esp.target_marker_color[" << i << "]=" << globals::esp::target_marker_color[i] << std::endl;
        }
        for (int i = 0; i < 3; ++i) {
            configFile << "esp.friend_box_color[" << i << "]=" << globals::esp::friend_box_color[i] << std::endl;
        }
        for (int i = 0; i < 3; ++i) {
            configFile << "esp.target_box_color[" << i << "]=" << globals::esp::target_box_color[i] << std::endl;
        }

        configFile.close();
    }

    void periodic_save() {
        while (true) {
            save_config();
            std::this_thread::sleep_for(std::chrono::milliseconds(250));
        }
    }
}
