namespace features
{
    struct tracer {
        Vector3 start_pos, end_pos;
        std::chrono::time_point<std::chrono::steady_clock> start_time;
        float initial_alpha = 255.0f;
        float current_alpha = 255.0f;
    };

    class Tracer_manager {
    public:
        void add_tracer(Vector3 start_pos, Vector3 end_pos) {
            for (const auto& existing_tracer : tracers) {
                if (existing_tracer.start_pos == start_pos || existing_tracer.end_pos == end_pos) {
                    return;
                }
            }
            tracer new_tracer{ start_pos, end_pos, std::chrono::steady_clock::now() };
            tracers.push_back(new_tracer);
        }

        void update_tracers() {
            auto now = std::chrono::steady_clock::now();
            for (auto& tracer : tracers) {
                auto elapsed = std::chrono::duration<float>(now - tracer.start_time).count();
                if (elapsed > globals::tracer_duration - 0.4f) {
                    float time_since_fade = elapsed - (globals::tracer_duration - 0.4f);
                    tracer.current_alpha = tracer.initial_alpha * (globals::tracer_duration - (time_since_fade / 0.2f));
                    if (tracer.current_alpha < 0.0f) {
                        tracer.current_alpha = 0.0f;
                    }
                }
            }
            tracers.erase(
                std::remove_if(tracers.begin(), tracers.end(), [&](const tracer& tracer) {
                    return tracer.current_alpha == 0.0f;
                    }),
                tracers.end()
            );
        }

        std::vector<tracer> get_tracers()
        {
            return tracers;
        }

    private:
        std::vector<tracer> tracers;
    };

    Tracer_manager tracer_manager;

    void tracers()
    {
        static Vector3 last_impact;
        static bool first = false;

        while (true)
        {
            Sleep(1);

            if (!game::world)
                continue;

            if (!game::local_player)
                continue;

            std::lock_guard<std::mutex> lock(globals::mutex::tracer_mutex);

            tracer_manager.update_tracers();

            if (GetAsyncKeyState(0x01))
            {
                uintptr_t weapon_manager = memory::Read<uintptr_t>(game::local_player + offsets::weapon_manager);

                if (!weapon_manager)
                    continue;

                uintptr_t v2 = memory::Read<uintptr_t>(weapon_manager + 0x78);

                if (!v2)
                    continue;

                uintptr_t v3 = memory::Read<uintptr_t>(v2 + offsets::c_weapon_offset);

                if (!v3)
                    continue;

                Vector3 start_pos = memory::Read<Vector3>(v3 + 0x20);
                Vector3 end_pos = memory::Read<Vector3>(weapon_manager + 0x1B0);

                if (!first)
                {
                    end_pos = last_impact;
                    first = true;
                }

                if (end_pos == last_impact)
                    continue;

                tracer_manager.add_tracer(start_pos, end_pos);

                last_impact = end_pos;
            }
        }
    }
}
