#include <psapi.h>

namespace memory
{
	using namespace std;


	inline std::uint32_t process_id = 0;
	inline HANDLE handle;
	inline MODULEENTRY32 mod_entry;

	struct HandleDisposer
	{
		using pointer = HANDLE;
		void operator()(HANDLE handle) const
		{
			if (handle != NULL || handle != INVALID_HANDLE_VALUE)
			{
				CloseHandle(handle);
			}
		}
	};

	using unique_handle = std::unique_ptr<HANDLE, HandleDisposer>;

	ULONG64 get_module_base_address(DWORD process_id, const char* module_name) {
		HANDLE hModuleSnap = CreateToolhelp32Snapshot(TH32CS_SNAPMODULE | TH32CS_SNAPMODULE32, process_id);
		if (hModuleSnap == INVALID_HANDLE_VALUE) {
			return 0;
		}

		MODULEENTRY32 me32;
		me32.dwSize = sizeof(MODULEENTRY32);

		if (!Module32First(hModuleSnap, &me32)) {
			CloseHandle(hModuleSnap);
			return 0;
		}

		do {
			if (_tcsicmp(me32.szModule, module_name) == 0) {
				CloseHandle(hModuleSnap);
				return (ULONG64)me32.modBaseAddr;
			}
		} while (Module32Next(hModuleSnap, &me32));

		CloseHandle(hModuleSnap);
		return 0;
	}

    template <class T>
    inline T Read(uintptr_t read_address)
    {
        T temp = {};

        driver::Request r;
        r.target = reinterpret_cast<PVOID>(read_address);
        r.buffer = &temp;
        r.size = sizeof(T);

        DeviceIoControl(handle, driver::codes::read, &r, sizeof(r), &r, sizeof(r), nullptr, nullptr);

        return temp;
    }

    template<typename S>
    inline void Write(uintptr_t write_address, const S& value)
    {
        driver::write_memory(handle, write_address, value);
    }

	std::uint32_t get_process_id(std::string_view process_name)
	{
		PROCESSENTRY32 processentry;
		const unique_handle snapshot_handle(CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, NULL));

		if (snapshot_handle.get() == INVALID_HANDLE_VALUE)
			return NULL;

		processentry.dwSize = sizeof(MODULEENTRY32);

		while (Process32Next(snapshot_handle.get(), &processentry) == TRUE)
		{
			if (process_name.compare(processentry.szExeFile) == NULL)
			{
				return processentry.th32ProcessID;
			}
		}
		return NULL;
	}

	int extract_number(const std::string& str) {
		int result = 0;

		for (char ch : str) {
			if (std::isdigit(ch)) {
				result = result * 10 + (ch - '0');
			}
		}

		return result;
	}

    MODULEENTRY32 get_module_entry(DWORD procId, const char* modName)
    {
        MODULEENTRY32 modEntry;
        HANDLE hSnap = CreateToolhelp32Snapshot(TH32CS_SNAPMODULE | TH32CS_SNAPMODULE32, procId);
        if (hSnap != INVALID_HANDLE_VALUE)
        {
            modEntry.dwSize = sizeof(modEntry);
            if (Module32First(hSnap, &modEntry))
            {
                do
                {
                    if (!strcmp(modEntry.szModule, modName))
                    {
                        break;
                    }
                } while (Module32Next(hSnap, &modEntry));
            }
        }
        CloseHandle(hSnap);
        return modEntry;
    }

	std::uint32_t get_current_build_number()
	{
		PROCESSENTRY32 processentry;
		const unique_handle snapshot_handle(CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, NULL));

		if (snapshot_handle.get() == INVALID_HANDLE_VALUE)
			return NULL;

		processentry.dwSize = sizeof(MODULEENTRY32);

		while (Process32Next(snapshot_handle.get(), &processentry) == TRUE)
		{
			std::string exeFileName = processentry.szExeFile;
			if (exeFileName.find("FiveM_b") == 0 &&
				exeFileName.rfind("_GTAProcess.exe") == exeFileName.length() - strlen("_GTAProcess.exe"))
			{
				return extract_number(exeFileName);
			}
		}
		return NULL;
	}
	bool compare_memory(const BYTE* bData, const BYTE* bMask, const char* szMask) {
		for (; *szMask; ++szMask, ++bData, ++bMask) {
			if (*szMask == 'x' && *bData != *bMask) {
				return false;
			}
		}
		return (*szMask == NULL);
	}

	DWORD64 find_signature(HANDLE TargetProcess, DWORD64 start, DWORD64 size, const char* sig, const char* mask)
	{
		BYTE* data = new BYTE[size];
		SIZE_T bytesRead;

		ReadProcessMemory(TargetProcess, (LPVOID)start, data, size, &bytesRead);

		for (int i = 0; i < size; i++)
		{
			if (compare_memory((const BYTE*)(data + i), (const BYTE*)sig, mask)) {
				return start + i;
			}
		}
		delete[] data;
		return NULL;
	}

	

    uint64_t get_actual_address(uint64_t addr, MODULEENTRY32 mod, int off)
    {
        uint32_t address = 0x0;
        address = (addr + Read<uint32_t>(addr) + off) - (uint64_t)(mod.modBaseAddr);
        return address;
    }

    std::string get_process_name_from_hwnd(HWND hwnd) {
        DWORD processId;
        GetWindowThreadProcessId(hwnd, &processId);

        HANDLE hProcess = OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, FALSE, processId);
        if (hProcess == NULL) {
            return "";
        }

        char buffer[MAX_PATH];
        if (GetModuleFileNameEx(hProcess, NULL, buffer, MAX_PATH) == 0) {
            CloseHandle(hProcess);
            return "";
        }

        CloseHandle(hProcess);
        return std::string(buffer);
    }

}
